#!/usr/bin/perl
#
# qmail-masq.pl v0.4 by Davide Giunchi (davidegiunchi@libero.it)
#
# LICENSE:
#
# Copyright (c) 2001 Davide Giunchi
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#

require '/etc/qmail-masq.conf';		# execute config file to import user settings

$cont=0;

while (<STDIN>) 
{
 if ($end_all==1) 
 {
  print;				# header finished so always print until the end of email
  next;
 }
 elsif (/^\n$|^\cM$/)                   # if there's only a newline in a line
 { 				        # it's sign that the headers are finished	
  &parse_header ();                   # call the subroutine to (eventually) change the 'From:' field
  for ($c=0;$c!=$cont+1;$c++)         # on the sub's exit print the headers
   {
    print "$head{$c}";
   }
  print;
  $end_all=1;
  next;
 }
 elsif (/^To:.*[^ \t\n\r\f<>]+?\@[^ \t\n\r\f<>,;]+.*?$/)  # found 'To:'
 {  	 	
  $head{$cont}=$_;
  $cont++;
  $found_to=1;
  while (/[^ \t\n\r\f<>]+\@([^ \t\n\r\f><,;]+)/g)
   {						# search in all To adressee if there's an external one
    $to_external=1 if ($1 ne $domain);
   }
  next;		
 }
 elsif (/^From:(.*?)([^ \t\n\r\f<>]+?)\@([^ \t\n\r\f<>,;]+)(.*)$/)
 {  						 # found 'From:'
  $nome_mitt=$1;	# memorize the name e and adress (diveided into user and domain) of the sender
  $user_mitt=$2;
  $domain_mitt=$3;
  $from_altro=$4;
  $cont_mitt=$cont;     # sign where the 'From:' has been found
  $head{$cont}=$_;
  $cont++;
  $found_from=1;
  next;
 }
 elsif (/^(BCC|CC):.*?[^ \t\n\r\f<>]+\@[^ \t\n\r\f<>;,]+.*$/i) 
 {  							# found CC or BCC
  $cc=1 if ($1 =~ /^CC$/i);
  $bcc=$cont if ($1 =~ /^BCC$/i);
  &control_cc();
  $head{$cont}=$_;
  $cont++;
  $cc=0;
  next;
 }							# take note of Return-Path header
 elsif (/^Return-Path: <.+\@.+>\n$/) 
 {
  $returnpath=$cont;
  $head{$cont}=$_;
  $cont++;
  next;
 }							# take note of Notified-to header
 elsif (/^Notified-to:.*/) 
 {
  $notified_to=$cont;
  $head{$cont}=$_;
  $cont++;
  next;
 }							# take note of Sender header
 elsif (/^Sender:.*/) 
 {
  $sender=$cont;
  $head{$cont}=$_;
  $cont++;
  next;
 }							# take note of X-Sender header
 elsif (/^X-Sender:.*/) 
 {
  $xsender=$cont;
  $head{$cont}=$_;
  $cont++;
  next;
 }
 else 
 {	       		# memorize all the headers
  $head{$cont}=$_;
  $cont++;
 }
}


sub parse_header
{
 SUB: {
 if ($domain_mitt ne $domain)    
 { 				  # if the sender is external don't modify nothing and exit from the cycle
    last SUB;
 }
 elsif ($to_external==1) #if the adressee is external and the sender is internal modify the "To:"field
 { 
    while (($expect_user, $expect_mail)=each(%masq))
    {					# parse the exeption hash, if the name on the hash it's equal to
      $user_masq=$expect_mail if ($user_mitt eq $expect_user); # the true adress of sender  
    }						            # place the variable with the user to masquerade
    $head{$cont_mitt}="From:$nome_mitt$user_masq$from_altro\n"; # modify the 'From:' field

    $head{$returnpath}="Return-Path: <$user_masq>\n"; # modify the 'Return-Path:' field

    delete $head{$notified_to} if ($notified_to); # delete "Notified-to", "Sender" and "X-Sender"
    delete $head{$sender} if ($sender);           # if they are present
    delete $head{$xsender} if ($xsender);         # (it could reveal your real internal addresses)
 }
 else 
 {
  last SUB;
 }
      }
}

sub control_cc
{
  while (/[^ \t\n\r\f<>]+\@([^ \t\n\r\f><,;]+)/g)
   {						# search in all CC adressee if there's an external one 
    $to_external=1 if ($1 ne $domain);
    $cc_internal=$cont if ($1 eq $domain && $cc==1); # there's an internal user in the CC (to put in the BCC)
   }						     # (TODO in the next qmail-version)
  $found_cc=1;					# sign that the CC has been found (usless)
}
